// *******************************************************************
//   (C) Copyright 2014 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Tablut (core)
// *******************************************************************
// FILE INFORMATION:
//   File:     move.h
//   Author:   Jonathan K. Vis
//   Revision: 1.01a
//   Date:     2014/01/20
// *******************************************************************
// DESCRIPTION:
//   Defines the move structure.
// *******************************************************************

#if !defined(__move_h__)
#define __move_h__

#include "pieces.h"
#include "square.h"


static int const MAX_MOVES = 256;
static int const MAX_MOVES_PER_PIECE = 16;


struct Move
{
  explicit inline Move(void) { }

  explicit inline Move(Square const &from, Square const &to):
                    from(from), to(to) { }


  bool is_capture(void) const;


  bool operator==(Move const &move) const;
  bool operator!=(Move const &move) const;


  Square from;
  Square to;
  int capture[4]; // 0 -- north, 1 -- east, 2 -- south, 3 -- west

}; // Move


inline bool Move::is_capture(void) const
{
  return capture[0] != NONE ||
         capture[1] != NONE ||
         capture[2] != NONE ||
         capture[3] != NONE;
} // Move::is_capture

inline bool Move::operator==(Move const &move) const
{
  return from == move.from && to == move.to;
} // Move::operator==

inline bool Move::operator!=(Move const &move) const
{
  return !(*this == move);
} // Move::operator!=

#endif

