// *******************************************************************
//   (C) Copyright 2014 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Tablut (engine)
// *******************************************************************
// FILE INFORMATION:
//   File:     main.cc
//   Author:   Jonathan K. Vis
//   Revision: 1.01a
//   Date:     2014/02/12
// *******************************************************************
// DESCRIPTION:
//   Entry point for a Tablut engine using ASGECP.
// *******************************************************************

#include "../core/position.h"
#include "../interface/cstdio_interface.h"
#include "../interface/jlkiss64.h"
#include "../interface/options.h"
#include "search.h"

#include <pthread.h>
#include <time.h>

#include <cstdlib>
#include <iostream>
using namespace std;


static char const* const ENGINE_NAME = "Generic Tablut Engine v0.1";
static int const MAX_DEPTH = 64;


typedef void* (*pthread_function)(void*);


volatile bool calculating = false;


void* search(void* parameters)
{
  static_cast<Search_Parameters*>(parameters)->value = iterative_deepening(*static_cast<Search_Parameters*>(parameters)->position,
                                                                            static_cast<Search_Parameters*>(parameters)->max_depth,
                                                                           *static_cast<Search_Parameters*>(parameters)->move);
  cout << "move ";
  write_move(*static_cast<Search_Parameters*>(parameters)->move, stdout);
  cout << endl;
  static_cast<Search_Parameters*>(parameters)->position->do_move(*static_cast<Search_Parameters*>(parameters)->move);
  char config[128] = {'\0'};
  get_configuration(*static_cast<Search_Parameters*>(parameters)->position, config);
  cout << "set config " << config << endl;
  terminate_search = true;
  calculating = false;
  pthread_exit(0);
} // search

void* timer(void* time)
{
  terminate_search = false;
  timespec req =
  {
    *static_cast<int*>(time) / 1000,
    (*static_cast<int*>(time) % 1000) * 1000000
  }; // req
  nanosleep(&req, 0);
  terminate_search = true;
  pthread_exit(0);
} // timer

int main(int argc, char* argv[])
{
  char const* option_depth = get_option(argc, argv, 'd');
  char const* option_time = get_option(argc, argv, 't');
  char const* configuration = get_option(argc, argv, 'p');

  int depth = option_depth == 0 ? MAX_DEPTH : atoi(option_depth);
  int time = option_time == 0 ? 0 : atoi(option_time);

  Position position;
  if (configuration != 0)
  {
    set_configuration(position, configuration);
  } // if
  else
  {
    position.initial();
  } // else

  pthread_t timer_thread = 0;
  pthread_t search_thread = 0;
  Move move;
  int value = 0;

  cin.rdbuf()->pubsetbuf(0, 0);
  cout.rdbuf()->pubsetbuf(0, 0);

  bool quiting = false;

  size_t const BUFFER_SIZE = 256;
  while (cin.good() && !quiting)
  {
    char buffer[BUFFER_SIZE] = {'\0'};
    cin.getline(buffer, BUFFER_SIZE);
    switch (buffer[0])
    {
      default:
        break;
      case 'Q':
      case 'q':
        quiting = true;
        break;
      case 'G':
      case 'g':
      {
        Move moves[MAX_MOVES];
        int const count = position.generate_moves(moves);
        char config[128] = {'\0'};
        get_configuration(position, config);
        cout << "set config " << config << endl;
        cout << "set moves ";
        for (int i = 0; i < count; ++i)
        {
          write_move(moves[i], stdout);
          cout << ' ';
        } // for
        cout << endl;
        break;
      } // case
      case 'C':
      case 'c':
        if (calculating && terminate_search)
        {
          calculating = false;
          pthread_join(search_thread, 0);
          pthread_cancel(timer_thread);
          pthread_join(timer_thread, 0);
        } // if
        if (position.is_won())
        {
          cout << "quit invalid" << endl;
          char config[128] = {'\0'};
          get_configuration(position, config);
          cout << "set config " << config << endl;
        } // if
        else if (!calculating)
        {
          calculating = true;
          terminate_search = false;
          if (time != 0)
          {
            pthread_create(&timer_thread, 0, timer, static_cast<void*>(&time));
          } // if
          Search_Parameters parameters = {&position, depth, &move, value};
          pthread_create(&search_thread, 0, search, static_cast<void*>(&parameters));
        } // if
        break;
      case 'K':
      case 'k':
        if (!calculating)
        {
          Move moves[MAX_MOVES];
          int const count = position.generate_moves(moves);
          if (count > 0)
          {
            int const random = RNG::next() % count;
            cout << "move ";
            write_move(moves[random], stdout);
            cout << endl;
            position.do_move(moves[random]);
            char config[128] = {'\0'};
            get_configuration(position, config);
            cout << "set config " << config << endl;
          } // if
        } // if
        terminate_search = true;
        break;
      case 'M':
      case 'm':
      {
        if (calculating)
        {
          pthread_cancel(search_thread);
          pthread_join(search_thread, 0);
          if (timer_thread != 0)
          {
            pthread_cancel(timer_thread);
            pthread_join(timer_thread, 0);
          } // if
          terminate_search = true;
          calculating = false;
        } // if
        size_t i = 1;
        while (i < BUFFER_SIZE - 4 && buffer[i] != '\n' && buffer[i] != ' ')
        {
          ++i;
        } // if
        Move input_move(Square(get_file(buffer[i + 1]), get_rank(buffer[i + 2])), Square(get_file(buffer[i + 3]), get_rank(buffer[i + 4])));
        if (position.is_legal_move(input_move))
        {
          position.do_move(input_move);
        } // if
        break;
      } // case
    } // switch
    if (calculating && terminate_search)
    {
      calculating = false;
      pthread_join(search_thread, 0);
      if (timer_thread != 0)
      {
        pthread_cancel(timer_thread);
        pthread_join(timer_thread, 0);
      } // if
    } // if
  } // while
  terminate_search = true;
  if (calculating)
  {
    calculating = false;
    pthread_join(search_thread, 0);
    if (timer_thread != 0)
    {
      pthread_cancel(timer_thread);
      pthread_join(timer_thread, 0);
    } // if
  } // if
  return 0;
} // main

