// *******************************************************************
//   (C) Copyright 2014 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Tablut (engine)
// *******************************************************************
// FILE INFORMATION:
//   File:     search.cc (depends on search.h)
//   Author:   Jonathan K. Vis
//   Revision: 1.01a
//   Date:     2014/02/03
// *******************************************************************
// DESCRIPTION:
//   Search functions for Tablut.
// *******************************************************************

#include "search.h"


volatile bool terminate_search = true;


int iterative_deepening(Position &position, int const max_depth, Move &move)
{
  static_cast<void>(move);

  int depth = 1;
  int value = perft(position, depth);

  // select a random move
  Move moves[MAX_MOVES];
  int const count = position.generate_moves(moves);
  if (count > 0)
  {
    move = moves[RNG::next() % count];
  } // if

  ++depth;
  while (!terminate_search && depth < max_depth)
  {
    value = perft(position, depth);
    ++depth;
  } // while
  return value;
} // iterative_deepening

uint64_t perft(Position &position, int const depth)
{
  if (terminate_search)
  {
    return -1;
  } // if
  if (depth <= 0 || position.is_won())
  {
    return 1;
  } // if
  Move moves[MAX_MOVES];
  int const count = position.generate_moves(moves);
  if (depth == 1)
  {
    return count;
  } // if
  uint64_t result = 0;
  for (int i = 0; i < count; ++i)
  {
    position.do_move(moves[i]);
    result += perft(position, depth - 1);
    position.undo_move(moves[i]);
  } // for
  return result;
} // perft

