// *******************************************************************
//   (C) Copyright 2014 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Tablut (interface)
// *******************************************************************
// FILE INFORMATION:
//   File:     cstio_interface.h
//   Author:   Jonathan K. Vis
//   Revision: 1.01a
//   Date:     2014/01/20
// *******************************************************************
// DESCRIPTION:
//   A basic cstdio interface.
// *******************************************************************

#if !defined(__cstdio_interface_h__)
#define __cstdio_interface_h__

#include "../core/location.h"
#include "../core/pieces.h"
#include "../core/position.h"
#include "../core/square.h"

#include <cstdio>


static char const FILE_SYMBOL[9] =
{
  'a',
  'b',
  'c',
  'd',
  'e',
  'f',
  'g',
  'h',
  'i'
}; // FILE_SYMBOL

static char const RANK_SYMBOL[9] =
{
  '9',
  '8',
  '7',
  '6',
  '5',
  '4',
  '3',
  '2',
  '1'
}; // RANK_SYMBOL

static char const CASTLE_SYMBOL = '#';
static char const SQUARE_SYMBOL = '.';
static char const CAPTURE_SYMBOL = 'x';
static char const SEPARATOR = '/';

static char const PIECE_SYMBOL[26] =
{
  '\0',
  'K',
  'S',
  'S',
  'S',
  'S',
  'S',
  'S',
  'S',
  'S',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm',
  'm'
}; // PIECE_SYMBOL


static inline int get_file(char const symbol)
{
  if (symbol == FILE_SYMBOL[FILE_A])
  {
    return FILE_A;
  } // if
  if (symbol == FILE_SYMBOL[FILE_B])
  {
    return FILE_B;
  } // if
  if (symbol == FILE_SYMBOL[FILE_C])
  {
    return FILE_C;
  } // if
  if (symbol == FILE_SYMBOL[FILE_D])
  {
    return FILE_D;
  } // if
  if (symbol == FILE_SYMBOL[FILE_E])
  {
    return FILE_E;
  } // if
  if (symbol == FILE_SYMBOL[FILE_F])
  {
    return FILE_F;
  } // if
  if (symbol == FILE_SYMBOL[FILE_G])
  {
    return FILE_G;
  } // if
  if (symbol == FILE_SYMBOL[FILE_H])
  {
    return FILE_H;
  } // if
  if (symbol == FILE_SYMBOL[FILE_I])
  {
    return FILE_I;
  } // if
  return -1;
} // get_file

static inline int get_rank(char const symbol)
{
  if (symbol == RANK_SYMBOL[RANK_1])
  {
    return RANK_1;
  } // if
  if (symbol == RANK_SYMBOL[RANK_2])
  {
    return RANK_2;
  } // if
  if (symbol == RANK_SYMBOL[RANK_3])
  {
    return RANK_3;
  } // if
  if (symbol == RANK_SYMBOL[RANK_4])
  {
    return RANK_4;
  } // if
  if (symbol == RANK_SYMBOL[RANK_5])
  {
    return RANK_5;
  } // if
  if (symbol == RANK_SYMBOL[RANK_6])
  {
    return RANK_6;
  } // if
  if (symbol == RANK_SYMBOL[RANK_7])
  {
    return RANK_7;
  } // if
  if (symbol == RANK_SYMBOL[RANK_8])
  {
    return RANK_8;
  } // if
  if (symbol == RANK_SYMBOL[RANK_9])
  {
    return RANK_9;
  } // if
  return -1;
} // get_rank

static inline void draw_board(Position const &position,
                              FILE*           stream = stderr)
{
  fputs("     ", stream);
  fputc(FILE_SYMBOL[FILE_A], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_B], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_C], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_D], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_E], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_F], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_G], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_H], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_I], stream);
  fputs("\n\n", stream);
  for (int rank = 0; rank < 9; ++rank)
  {
    fputs("  ", stream);
    fputc(RANK_SYMBOL[rank], stream);
    fputs("  ", stream);
    for (int file = 0; file < 9; ++file)
    {
      if (position.get_board(rank, file) == NONE)
      {
        if (Square(file, rank) == CASTLE)
        {
          fputc(CASTLE_SYMBOL, stream);
        } // if
        else
        {
          fputc(SQUARE_SYMBOL, stream);
        } // else
      } // if
      else
      {
        fputc(PIECE_SYMBOL[position.get_board(rank, file)], stream);
      } // else
      fputc(' ', stream);
    } // for
    fputc(' ', stream);
    fputc(RANK_SYMBOL[rank], stream);
    fputc('\n', stream);
  } // for
  fputs("\n     ", stream);
  fputc(FILE_SYMBOL[FILE_A], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_B], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_C], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_D], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_E], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_F], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_G], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_H], stream);
  fputc(' ', stream);
  fputc(FILE_SYMBOL[FILE_I], stream);
  fputc('\n', stream);
  return;
} // draw_board

static inline void write_move(Move const &move,
                              FILE*       stream = stderr)
{
  fputc(FILE_SYMBOL[move.from.file], stream);
  fputc(RANK_SYMBOL[move.from.rank], stream);
  fputc(FILE_SYMBOL[move.to.file], stream);
  fputc(RANK_SYMBOL[move.to.rank], stream);
  if (move.capture[0] != NONE)
  {
    fputc(CAPTURE_SYMBOL, stream);
    fputc(FILE_SYMBOL[move.to.file], stream);
    fputc(RANK_SYMBOL[move.to.rank - 1], stream);
  } // if
  if (move.capture[1] != NONE)
  {
    fputc(CAPTURE_SYMBOL, stream);
    fputc(FILE_SYMBOL[move.to.file + 1], stream);
    fputc(RANK_SYMBOL[move.to.rank], stream);
  } // if
  if (move.capture[2] != NONE)
  {
    fputc(CAPTURE_SYMBOL, stream);
    fputc(FILE_SYMBOL[move.to.file], stream);
    fputc(RANK_SYMBOL[move.to.rank + 1], stream);
  } // if
  if (move.capture[3] != NONE)
  {
    fputc(CAPTURE_SYMBOL, stream);
    fputc(FILE_SYMBOL[move.to.file - 1], stream);
    fputc(RANK_SYMBOL[move.to.rank], stream);
  } // if
  return;
} // write_move

static inline int get_configuration(Position const &position,
                                    char            string[128])
{
  string[0] = static_cast<char>(position.get_turn() % 2 + '0');
  int p = 1;
  for (int rank = 0; rank < 9; ++rank)
  {
    string[p] = SEPARATOR;
    ++p;
    int count = 0;
    for (int file = 0; file < 9; ++file)
    {
      if (position.get_board(rank, file) == NONE)
      {
        ++count;
      } // if
      else
      {
        if (count > 0)
        {
          string[p] = static_cast<char>(count + '0');
          ++p;
          count = 0;
        } // if
        string[p] = PIECE_SYMBOL[position.get_board(rank, file)];
        ++p;
      } // else
    } // for
  } // for
  string[p] = '/';
  string[p + 1] = FILE_SYMBOL[position.get_last_move().file];
  string[p + 2] = RANK_SYMBOL[position.get_last_move().rank];
  string[p + 3] = '\0';
  return p + 4;
} // get_configuration

static inline void set_configuration(Position  &position,
                                     char const string[128])
{
  position.clear();
  if (string[0] == '1')
  {
    position.do_null_move();
  } // if
  int p = 2;
  int rank = 0;
  int file = 0;
  int white_pawn = WHITE_PAWN_0;
  int black_pawn = BLACK_PAWN_0;
  while (rank < 9)
  {
    if (string[p] >= '0' && string[p] <= '9')
    {
      file += static_cast<int>(string[p] - '0');
    } // if
    else if (string[p] == SEPARATOR)
    {
      ++rank;
      file = 0;
    } // if
    else if (string[p] == PIECE_SYMBOL[WHITE_KING])
    {
      position.set_piece(WHITE_KING, Square(file, rank));
      ++file;
    } // if
    else if (string[p] == PIECE_SYMBOL[WHITE_PAWN_0])
    {
      position.set_piece(white_pawn, Square(file, rank));
      ++white_pawn;
      ++file;
    } // if
    else if (string[p] == PIECE_SYMBOL[BLACK_PAWN_0])
    {
      position.set_piece(black_pawn, Square(file, rank));
      ++black_pawn;
      ++file;
    } // if
    ++p;
  } // while
  position.set_last_move(Square(get_file(string[p]), get_rank(string[p + 1])));
  return;
} // set_configuration

#endif

