// *******************************************************************
//   (C) Copyright 2013 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Tablut (interface)
// *******************************************************************
// FILE INFORMATION:
//   File:     jlkiss64.h
//   Author:   Jonathan K. Vis
//   Revision: 1.01b
//   Date:     2014/02/03
// *******************************************************************
// DESCRIPTION:
//   64-bits random number generator based on: ``The KISS generator'',
//   G. Marsaglia and A. Zaman, 1993.
// *******************************************************************

#if !defined(__jlkiss64_h__)
#define __jlkiss64_h__

typedef unsigned int       uint32_t;
typedef unsigned long long uint64_t;


namespace {

class RNG
{
  public:
    static RNG next;


    static void seed(uint64_t const n);


    uint64_t operator()(void);


  private:
    inline RNG(void);


    static uint64_t x;
    uint64_t y;
    uint32_t z1;
    uint32_t c1;
    uint32_t z2;
    uint32_t c2;

}; // RNG

uint64_t RNG::x = 123456789123ull;


inline void RNG::seed(uint64_t const n)
{
  x = RNG::next() ^ n;
  return;
} // RNG::seed


inline uint64_t RNG::operator()(void)
{
  uint64_t t;

  x = 1490024343005336237ull * x + 123456789;
  y ^= y << 21;
  y ^= y >> 17;
  y ^= y << 30;
  t = 4294584393ull * z1 + c1;
  c1 = t >> 32;
  z1 = t;
  t = 4246477509ull * z2 + c2;
  c2 = t >> 32;
  z2 = t;
  return x + y + static_cast<uint64_t>(z1) +
         (static_cast<uint64_t>(z2) << 32);
} // RNG::operator()


inline RNG::RNG(void)
{
  y = 987654321987ull;
  z1 = 43219876;
  c1 = 6543217;
  z2 = 21987643;
  c2 = 1732654;
} // RNG::RNG


RNG RNG::next;

} // namespace

#endif

