// *******************************************************************
//   (C) Copyright 2014 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Tablut (referee)
// *******************************************************************
// FILE INFORMATION:
//   File:     ipc.cc (depends on ipc.h)
//   Author:   Jonathan K. Vis
//   Revision: 1.01a
//   Date:     2014/02/12
// *******************************************************************
// DESCRIPTION:
//   Defines the Inter-process Communication (IPC) interface.
// NOTE: this file is POSIX dependent.
// *******************************************************************

#include "ipc.h"

#include <cstdio>


// launches a client process with its STDIN/STDOUT bound to the
// caller. No error checking is performed.
void IPC_Interface::launch(char const* const arguments[])
{
  int p2c[2];
  int c2p[2];

  {
    int const result = pipe(p2c);
    if (result < 0)
    {
      return;
    } // if
  }
  {
    int const result = pipe(c2p);
    if (result < 0)
    {
      return;
    } // if
  }

  pID = fork();
  if (pID == 0)
  {
    close(p2c[1]);
    close(c2p[0]);

    dup2(p2c[0], STDIN_FILENO);
    dup2(c2p[1], STDOUT_FILENO);
    if (p2c[0] > 1)
    {
      close(p2c[0]);
    } // if
    close(c2p[1]);

    execv(arguments[0], const_cast<char* const* const>(arguments));
    fprintf(stderr, "ERROR\n");
    exit(-1);
  } // if
  close(p2c[0]);
  close(c2p[1]);
  fdIn = c2p[0];
  fdOut = p2c[1];
  return;
} // IPC_Interface::launch

size_t IPC_Interface::get_line(char* const  buffer,
                               size_t const size,
                               char const   delimiter) const
{
  size_t count = 0;

  while (count < size &&
         ::read(fdIn, &buffer[count], 1) > 0 &&
         buffer[count] != delimiter)
  {
    ++count;
  } // while
  return count + 1;
} // IPC_Interface::get_line

