// *******************************************************************
//   (C) Copyright 2014 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Tablut (referee)
// *******************************************************************
// FILE INFORMATION:
//   File:     ipc.h (implemented in ipc.cc)
//   Author:   Jonathan K. Vis
//   Revision: 1.01a
//   Date:     2014/02/11
// *******************************************************************
// DESCRIPTION:
//   Defines the Inter-process Communication (IPC) interface.
// NOTE: this file is POSIX dependent.
// *******************************************************************

#if !defined(__ipc_h__)
#define __ipc_h__

#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>


#include <cstdlib>


class IPC_Interface
{
  public:
    explicit IPC_Interface(void);
    ~IPC_Interface(void);

    void launch(char const* const arguments[]);
    void terminate(void);

    ssize_t read(char* const buffer, size_t const size) const;
    ssize_t write(char const* const buffer, size_t const size) const;

    size_t get_line(char* const  buffer,
                    size_t const size,
                    char const   delimiter = '\n') const;

  private:
    pid_t pID;
    int   fdIn;
    int   fdOut;

}; // IPC_Interface


// Default constructor provides an interface to STDIN/STDOUT
inline IPC_Interface::IPC_Interface(void):
  pID(-1),
  fdIn(STDIN_FILENO),
  fdOut(STDOUT_FILENO)
{
  // intentionally empty
} // IPC_Interface::IPC_Interface

inline IPC_Interface::~IPC_Interface(void)
{
  terminate();
} // IPC_Interface::~IPC_Interface


// Closes both pipes and waits for the client to terminate
inline void IPC_Interface::terminate(void)
{
  if (pID != -1)
  {
    close(fdIn);
    close(fdOut);
    waitpid(pID, 0, 0);
  } // if
  pID = -1;
  fdIn = STDIN_FILENO;
  fdOut = STDOUT_FILENO;
  return;
} // IPC_Interface::terminate


inline ssize_t IPC_Interface::read(char* const  buffer,
                                   size_t const size) const
{
  return ::read(fdIn, buffer, size);
} // IPC_Interface::read

inline ssize_t IPC_Interface::write(char const* const buffer,
                                    size_t const      size) const
{
  return ::write(fdOut, buffer, size);
} // IPC_Interface::write

#endif

