// *******************************************************************
//   (C) Copyright 2014 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Tablut (referee)
// *******************************************************************
// FILE INFORMATION:
//   File:     main.cc
//   Author:   Jonathan K. Vis
//   Revision: 1.01a
//   Date:     2014/02/11
// *******************************************************************
// DESCRIPTION:
//   A simple Tablut referee using IPC/ASGECP.
// *******************************************************************

#include "../core/position.h"
#include "../interface/cstdio_interface.h"
#include "../interface/options.h"
#include "ipc.h"

#include <cstdio>
#include <cstring>


static int const MAX_ARGS = 16;
static int const WHITE = 0;
static int const BLACK = 1;


int main(int argc, char* argv[])
{
  char* configuration = get_option(argc, argv, 'p');
  IPC_Interface child[2];
  {
    char* arguments[MAX_ARGS] = {strtok(get_option(argc, argv, 'w'), " ")};
    int i = 0;
    while (arguments[i] != 0 && i < MAX_ARGS - 2)
    {
      ++i;
      arguments[i] = strtok(0, " ");
    } // while
    if (i > 0 && arguments[0][0] != '0')
    {
      if (configuration != 0)
      {
        arguments[i] = const_cast<char*>("-p");
        arguments[i + 1] = configuration;
      } // if
      child[WHITE].launch(arguments);
    } // if
  }
  {
    char* arguments[MAX_ARGS] = {strtok(get_option(argc, argv, 'b'), " ")};
    int i = 0;
    while (arguments[i] != 0 && i < MAX_ARGS - 2)
    {
      ++i;
      arguments[i] = strtok(0, " ");
    } // while
    if (i > 0 && arguments[0][0] != '0')
    {
      if (configuration != 0)
      {
        arguments[i] = const_cast<char*>("-p");
        arguments[i + 1] = configuration;
      } // if
      child[BLACK].launch(arguments);
    } // if
  }

  Position position;
  if (configuration != 0)
  {
    set_configuration(position, configuration);
  } // if
  else
  {
    position.initial();
  } // else

  size_t const BUFFER_SIZE = 256;

  draw_board(position);
  while (!position.is_won())
  {
    child[position.get_turn() % 2].write("calculate\n", 10);
    char buffer[BUFFER_SIZE] = {'\0'};
    size_t const c = child[position.get_turn() % 2].get_line(buffer, BUFFER_SIZE);
    switch (buffer[0])
    {
      default:
        break;
      case 'M':
      case 'm':
      {
        size_t i = 1;
        while (i < BUFFER_SIZE - 4 && buffer[i] != '\n' && buffer[i] != ' ')
        {
          ++i;
        } // while
        Move input_move(Square(get_file(buffer[i + 1]), get_rank(buffer[i + 2])), Square(get_file(buffer[i + 3]), get_rank(buffer[i + 4])));
        if (position.is_legal_move(input_move))
        {
          position.do_move(input_move);
          child[position.get_turn() % 2].write(buffer, c);
          draw_board(position);
          write_move(input_move, stdout);
          putc('\n', stdout);
        } // if
        else
        {
          child[position.get_turn() % 2].write("quit illegal_move\n", 18);
          child[(position.get_turn() + 1) % 2].write("quit won\n", 9);
          child[WHITE].terminate();
          child[BLACK].terminate();
          return 1;
        } // else
        break;
      } // case
    } // switch
  } // while
  child[position.get_turn() % 2].write("quit lost\n", 10);
  child[(position.get_turn() + 1) % 2].write("quit won\n", 9);
  child[WHITE].terminate();
  child[BLACK].terminate();
  return 0;
} // main

